@@ -92,6 +92,9 @@ function getDownload(aID)
 
 function downloadCompleted(aDownload)
 {
+  // The download is changing state, so update the clear list button
+  updateClearListButton();
+
   // Wrap this in try...catch since this can be called while shutting down...
   // it doesn't really matter if it fails then since well.. we're shutting down
   // and there's no UI to update!
@@ -310,6 +313,28 @@ function copySourceLocation(aDownload)
   }
 }
 
+/**
+ * Remove the currently shown downloads from the download list.
+ */
+function clearDownloadList() {
+  // Clear the whole list if there's no search
+  if (gSearchTerms == "") {
+    gDownloadManager.cleanUp();
+    return;
+  }
+
+  // Remove each download starting from the end until we hit a download
+  // that is in progress
+  let item;
+  while ((item = gDownloadsView.lastChild) && !item.inProgress)
+    removeDownload(item);
+
+  // Clear the input as if the user did it and move focus to the list
+  gSearchBox.value = "";
+  gSearchBox.doCommand();
+  gDownloadsView.focus();
+}
+
 // This is called by the progress listener.
 var gLastComputedMean = -1;
 var gLastActiveDownloads = 0;
@@ -456,7 +481,7 @@ var gContextMenus = [
     , "menuitem_openReferrer"
     , "menuitem_copyLocation"
     , "menuseparator"
-    , "menuitem_clearList"
+    , "menuitem_selectAll"
   ],
   // DOWNLOAD_FINISHED
   [
@@ -466,8 +491,9 @@ var gContextMenus = [
     , "menuitem_openReferrer"
     , "menuitem_copyLocation"
     , "menuseparator"
+    , "menuitem_selectAll"
+    , "menuseparator"
     , "menuitem_removeFromList"
-    , "menuitem_clearList"
   ],
   // DOWNLOAD_FAILED
   [
@@ -476,8 +502,9 @@ var gContextMenus = [
     , "menuitem_openReferrer"
     , "menuitem_copyLocation"
     , "menuseparator"
+    , "menuitem_selectAll"
+    , "menuseparator"
     , "menuitem_removeFromList"
-    , "menuitem_clearList"
   ],
   // DOWNLOAD_CANCELED
   [
@@ -486,8 +513,9 @@ var gContextMenus = [
     , "menuitem_openReferrer"
     , "menuitem_copyLocation"
     , "menuseparator"
+    , "menuitem_selectAll"
+    , "menuseparator"
     , "menuitem_removeFromList"
-    , "menuitem_clearList"
   ],
   // DOWNLOAD_PAUSED
   [
@@ -499,7 +527,7 @@ var gContextMenus = [
     , "menuitem_openReferrer"
     , "menuitem_copyLocation"
     , "menuseparator"
-    , "menuitem_clearList"
+    , "menuitem_selectAll"
   ],
   // DOWNLOAD_QUEUED
   [
@@ -510,15 +538,16 @@ var gContextMenus = [
     , "menuitem_openReferrer"
     , "menuitem_copyLocation"
     , "menuseparator"
-    , "menuitem_clearList"
+    , "menuitem_selectAll"
   ],
   // DOWNLOAD_BLOCKED_PARENTAL
   [
     "menuitem_openReferrer"
     , "menuitem_copyLocation"
     , "menuseparator"
+    , "menuitem_selectAll"
+    , "menuseparator"
     , "menuitem_removeFromList"
-    , "menuitem_clearList"
   ],
   // DOWNLOAD_SCANNING
   [
@@ -527,23 +556,25 @@ var gContextMenus = [
     , "menuitem_openReferrer"
     , "menuitem_copyLocation"
     , "menuseparator"
-    , "menuitem_clearList"
+    , "menuitem_selectAll"
   ],
   // DOWNLOAD_DIRTY
   [
     "menuitem_openReferrer"
     , "menuitem_copyLocation"
     , "menuseparator"
+    , "menuitem_selectAll"
+    , "menuseparator"
     , "menuitem_removeFromList"
-    , "menuitem_clearList"
   ],
   // DOWNLOAD_BLOCKED_POLICY
   [
     "menuitem_openReferrer"
     , "menuitem_copyLocation"
     , "menuseparator"
+    , "menuitem_selectAll"
+    , "menuseparator"
     , "menuitem_removeFromList"
-    , "menuitem_clearList"
   ]
 ];
 
@@ -615,21 +646,20 @@ var gDownloadDNDObserver =
 var gDownloadViewController = {
   isCommandEnabled: function(aCommand, aItem)
   {
-    // This switch statement is for commands that do not need a download object
-    switch (aCommand) {
-      case "cmd_clearList":
-        return gDownloadManager.canCleanUp;
-    }
-
     let dl = aItem;
     let download = null; // used for getting an nsIDownload object
 
     switch (aCommand) {
       case "cmd_cancel":
         return dl.inProgress;
-      case "cmd_open":
+      case "cmd_open": {
         let file = getLocalFileFromNativePathOrUrl(dl.getAttribute("file"));
         return dl.openable && file.exists();
+      }
+      case "cmd_show": {
+        let file = getLocalFileFromNativePathOrUrl(dl.getAttribute("file"));
+        return file.exists();
+      }
       case "cmd_pause":
         download = gDownloadManager.getDownload(dl.getAttribute("dlid"));
         return dl.inProgress && !dl.paused && download.resumable;
@@ -644,7 +674,6 @@ var gDownloadViewController = {
       case "cmd_removeFromList":
       case "cmd_retry":
         return dl.removable;
-      case "cmd_show":
       case "cmd_copyLocation":
         return true;
     }
@@ -691,30 +720,6 @@ var gDownloadViewController = {
     cmd_copyLocation: function(aSelectedItem) {
       copySourceLocation(aSelectedItem);
     },
-    cmd_clearList: function() {
-      // If we're performing all, we can save some work by only doing it once
-      if (gPerformAllCallback === null)
-        gPerformAllCallback = function() {};
-      else if (gPerformAllCallback)
-        return;
-
-      // Clear the whole list if there's no search
-      if (gSearchTerms == "") {
-        gDownloadManager.cleanUp();
-      }
-      else {
-        // Remove each download starting from the end until we hit a download
-        // that is in progress
-        let item;
-        while ((item = gDownloadsView.lastChild) && !item.inProgress)
-          removeDownload(item);
-
-        // Clear the input as if the user did it and move focus to the list
-        gSearchBox.value = "";
-        gSearchBox.doCommand();
-        gDownloadsView.focus();
-      }
-    }
   }
 };
 
@@ -1037,6 +1042,9 @@ function removeFromView(aDownload)
 
   // Color everything after from the newly selected item
   stripeifyList(gDownloadsView.selectedItem);
+
+  // We might have removed the last item, so update the clear list button
+  updateClearListButton();
 }
 
 function getReferrerOrSource(aDownload)
@@ -1096,6 +1104,9 @@ function buildDownloadList(aForceBuild)
     // Start building the list and select the first item
     stepListBuilder(1);
     gDownloadsView.selectedIndex = 0;
+
+    // We just tried to add a single item, so we probably need to enable
+    updateClearListButton();
   }, 0);
 }
 
@@ -1111,10 +1122,11 @@ function stepListBuilder(aNumItems) {
   try {
     // If we're done adding all items, we can quit
     if (!gStmt.executeStep()) {
-      // Send a notification that we finished
-      Cc["@mozilla.org/observer-service;1"].
-      getService(Ci.nsIObserverService).
-      notifyObservers(window, "download-manager-ui-done", null);
+      // Send a notification that we finished, but wait for clear list to update
+      updateClearListButton();
+      setTimeout(function() Cc["@mozilla.org/observer-service;1"].
+        getService(Ci.nsIObserverService).
+        notifyObservers(window, "download-manager-ui-done", null), 0);
 
       return;
     }
@@ -1206,6 +1218,9 @@ function prependList(aDownload)
     // Because of the joys of XBL, we can't update the buttons until the
     // download object is in the document.
     updateButtons(item);
+
+    // We might have added an item to an empty list, so update button
+    updateClearListButton();
   }
 }
 
@@ -1279,3 +1294,14 @@ function getLocalFileFromNativePathOrUrl(aPathOrUrl)
     return f;
   }
 }
+
+/**
+ * Update the disabled state of the clear list button based on whether or not
+ * there are items in the list that can potentially be removed.
+ */
+function updateClearListButton()
+{
+  let button = document.getElementById("clearListButton");
+  // The button is enabled if we have items in the list and we can clean up
+  button.disabled = !(gDownloadsView.itemCount && gDownloadManager.canCleanUp);
+}
